'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function () {
  var isSandboxed = arguments.length > 1;
  var sandboxName = isSandboxed ? arguments.length <= 0 ? undefined : arguments[0] : undefined;
  var actions = isSandboxed ? arguments.length <= 1 ? undefined : arguments[1] : arguments.length <= 0 ? undefined : arguments[0];

  if (sandboxName && typeof sandboxName === 'string' && !sandboxName.length) {
    throw new Error('Sandboxed states names must be a valid string');
  }

  var initialState = actions.initial;
  delete actions.initial;

  var namedActions = {};

  var currentState = initialState;

  var reducerWithActions = function reducerWithActions() {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
    var action = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    if (namedActions[action.type]) {
      var nextState = namedActions[action.type](state, action.payload);

      currentState = Object.assign({}, state, nextState);
    }
    return currentState;
  };

  Object.keys(actions).forEach(function (actionName) {
    var resolvedActionName = sandboxName ? sandboxName + '_' + actionName : actionName;

    namedActions[resolvedActionName] = actions[actionName];

    var actionMethod = function actionMethod(payload) {
      var action = {
        type: resolvedActionName,
        payload: payload
      };

      return (0, _middleware.dispatch)(action);
    };

    reducerWithActions[actionName] = actionMethod;

    if (!sandboxName) {
      (0, _actions.addAction)(actionName, actionMethod);
    }

    if (typeof process !== 'undefined' && process.env.NODE_ENV === 'testing') {
      reducerWithActions['_' + actionName] = actions[actionName];
    }
  });

  return reducerWithActions;
};

var _middleware = require('./middleware');

var _actions = require('./actions');